/* 
 * Anti-aliased line draw. 
 * 
 */

#include "AAline.h"
#include "draw.h"
#include "math.h"
#include <stdlib.h>

int s_gamma[256];

void initGamma(void)
{
  int i;
  double gamma = 2.7;
  double scale = 32.7513346400311235975901565335;


  // i = p ^ gamma
  // n = v ^ (1/gamma)
  // logn = logn(v)/gamma
  // 255 / (255 ^ 0.4) = 255 / 9.1752112487580066273279890797923
  // 

  for (i = 0; i < 256; i++)
  {
    s_gamma[i] = (int)(scale * exp(log((double)i) / gamma) + 0.5);
  }
}  

// This one based on Video Demystified book
void initGamma2(void)
{
  int i;

  for (i = 0; i < 256; i++)
  {
      s_gamma[i] = (int)(255*exp(log(((double)i)/255)*0.45));
  }
}  

//void Plot(CDC *pDC, int x, int y, int r, int g, int b)
//{
// COLORREF cref;
//
//  cref = RGB(s_gamma[r], s_gamma[g], s_gamma[b]);
//  pDC->SetPixel(x, y, cref);
//}

void Plot(short* frameBuffer, int x, int y, int r, int g, int b)
{
  int pixel;

  pixel = ((s_gamma[r]>>3)<<11) | ((s_gamma[g]>>2)<<5) | (s_gamma[b]>>3);
  vga_set_pixel(x, y, pixel, frameBuffer);
}

void DrawLineAA(short* frameBuffer, TPOINT from, TPOINT to)
{
  int c, x, y, xd, yd, xp, yp, ep, nep, err, x2, y2;
  int xinc, yinc, einc, pixCount;
  int xl = from.x << TSHIFT, yl=from.y << TSHIFT, xh=to.x << TSHIFT, yh=to.y << TSHIFT;
  int vert;
  
  xd = abs(from.x - to.x);
  yd = abs(from.y - to.y);

  vert = (yd > xd);

  if (vert)
  {
    // more vertical than horizontal, y moves faster than x, so increment y each time
    xinc = (xh - xl) / yd;
    yinc = (yh > yl) ? TSCALE : -TSCALE;
    einc = xinc;
  }
  else
  {
    // more horizontal than vertical, x moves faster than y, so increment x each time
    xinc = (xh > xl) ?  TSCALE : -TSCALE;
    yinc = (yh - yl) / xd;
    einc = yinc;
  }

  x = xl;
  y = yl;
  err = 0;
  pixCount = vert ? yd : xd;
  x2 = vert;
  y2 = !vert;

  for (c = 0; c < pixCount; c++)
  {
    xp = x >> TSHIFT;
    yp = y >> TSHIFT;
    ep = err & TMASK;
    nep = (TSCALE-ep);

    // perform anti-aliasing based on error term
    Plot(frameBuffer, xp,  yp,  from.r*nep >> TSHIFT, from.g*nep >> TSHIFT, from.b*nep >> TSHIFT);
    Plot(frameBuffer, xp+x2, yp+y2, from.r*ep >> TSHIFT,  from.g*ep >> TSHIFT,  from.b*ep >> TSHIFT);

    // non-aliased
    //Plot(frameBuffer, xp,  yp,  from.r, from.g, from.b);

    x += xinc;
    y += yinc;
    err += einc;
  }
}

//main()
//{
// initGamma();
//  TPOINT s = {900,100,255,255,255};
// TPOINT d = {10,50,100,100,100};
// for (int i = 0; i < 1000; i++)
//  {
//
//   DrawLineAA(pDC, s, d);
//
//    s.x = d.x;//rand() % 1000;
//    s.y = d.y;//rand() % 1000;
//    d.x = rand() % 1000;
//    d.y = rand() % 1000;
//    s.r = rand() % 255;
//   s.g = rand() % 255;
//   s.b = rand() % 255;
// }
//}

