// bmpto565.cpp : Convert .bmp to 565 format in a 16 bit frame buffer
//

#include <stdio.h>
#include "string.h"
#include "lcd.h"

//#define DEBUG

typedef struct {
   unsigned short int type;             // Magic identifier
   unsigned int size;                   // File size in bytes
   unsigned int reserved;
   unsigned int offset;                 // Offset to image data, bytes
} HEADER;

typedef struct {
   unsigned int infoSize;				// Header size in bytes
   int width,height;					// Width and height of image
   unsigned short int planes;			// Number of colour planes
   unsigned short int bits;				// Bits per pixel
   unsigned int compression;			// Compression type
   unsigned int imageSize;				// Image size in bytes
   int xPixelsPerM,yPixelsPerM;			// Pixels per meter
   unsigned int coloursUsed;			// Number of colours
   unsigned int coloursImportant;		// Important colours
   unsigned int palletteR;				// pallette
   unsigned int palletteG;				// pallette
   unsigned int palletteB;				// pallette

} INFOHEADER;

/*
 * Read a 16 bit bitmap (in 565 format) into a framebuffer
 */
int bmpTo565(FILE *bmpFile, short * frameBuffer) {
	HEADER header;
	INFOHEADER infoHeader;

  // Read the file header
#ifdef DEBUG
	printf("\nReading file header...\n");
#endif
  fread(&header.type, sizeof(header.type), 1, bmpFile);
  fread(&header.size, sizeof(header.size), 1, bmpFile);
  fread(&header.reserved, sizeof(header.reserved), 1, bmpFile);
  fread(&header.offset, sizeof(header.offset), 1, bmpFile);

#ifdef DEBUG
	printf("%04X\n", header.type);
	printf("File Size: %i bytes\n", header.size);
	printf("Reserved is %i\n", header.reserved);
	printf("Data Offset: %i bytes\n", header.offset);
#endif

	// Check that it's a valid bitmap file, header type = "BM"
  if (header.type != 19778) {
    fclose(bmpFile);
	printf("Sorry, the file does not appear to be a BMP\n");
    return -1;
  }

  // Read the bitmap info header */
#ifdef DEBUG
	printf("\nReading bitmap info header...\n");
#endif
  fread(&infoHeader.infoSize, sizeof(infoHeader.infoSize), 1, bmpFile);
  fread(&infoHeader.width, sizeof(infoHeader.width), 1, bmpFile);
  fread(&infoHeader.height, sizeof(infoHeader.height), 1, bmpFile);
  fread(&infoHeader.planes, sizeof(infoHeader.planes), 1, bmpFile);
  fread(&infoHeader.bits, sizeof(infoHeader.bits), 1, bmpFile);
  fread(&infoHeader.compression, sizeof(infoHeader.compression), 1, bmpFile);
  fread(&infoHeader.imageSize, sizeof(infoHeader.imageSize), 1, bmpFile);
  fread(&infoHeader.xPixelsPerM, sizeof(infoHeader.xPixelsPerM), 1, bmpFile);
  fread(&infoHeader.yPixelsPerM, sizeof(infoHeader.yPixelsPerM), 1, bmpFile);
  fread(&infoHeader.coloursUsed, sizeof(infoHeader.coloursUsed), 1, bmpFile);
  fread(&infoHeader.coloursImportant, sizeof(infoHeader.coloursImportant), 1, bmpFile);
  fread(&infoHeader.palletteR, sizeof(infoHeader.palletteR), 1, bmpFile);
  fread(&infoHeader.palletteG, sizeof(infoHeader.palletteG), 1, bmpFile);
  fread(&infoHeader.palletteB, sizeof(infoHeader.palletteB), 1, bmpFile);

#ifdef DEBUG
	printf("Info header size: %i bytes\n", infoHeader.infoSize);
	printf("Image width: %i pixels\n", infoHeader.width);
	printf("Image height %i pixels\n", infoHeader.height);
	printf("Number of planes: %i\n", infoHeader.planes);
	printf("Bits/pixel: %i\n", infoHeader.bits);
	printf("Compression: %i bytes\n", infoHeader.compression);
	printf("Compressed size: %i\n", infoHeader.imageSize);
	printf("Horizontal resolution: %i pixels/m\n", infoHeader.xPixelsPerM);
	printf("Vertical resolution: %i pixels/m\n", infoHeader.yPixelsPerM);
	printf("Colours used: %i\n", infoHeader.coloursUsed);
	printf("Impportant colours: %i bytes\n", infoHeader.coloursImportant);
	printf("Pallette R Mask: %x\n", infoHeader.palletteR);
	printf("Pallette G Mask: %x\n", infoHeader.palletteG);
	printf("Pallette B Mask: %x\n", infoHeader.palletteB);
#endif

	if (infoHeader.bits != 16) {
    fclose(bmpFile);
		printf("\nUnsupported bit count: %i bits/pixel\n", infoHeader.bits);
		return -1;
	}

	if (infoHeader.compression != 3) {
    fclose(bmpFile);
		printf("\nUnsupported compression Mode: %i\n", infoHeader.compression);
		return -1;
	}

	long seekPos;

	// Seek to start of bitmap data
	seekPos = header.offset;
	fseek(bmpFile, seekPos, SEEK_SET);

  fread(frameBuffer, FRAMEBUFFERSIZE*2, 1, bmpFile);

	return 0;
}

