/*
 * This version is for the Automotive demo.
 * 
 * Frame buffers are accessed as arrays rather than memory
 */
 
#include <stdio.h>
#include "auto32.h"
#include "system.h"
#include "lcd.h"
#include "alt_types.h"

#include <stdlib.h>
#include <unistd.h>
#include "draw.h"
#include "string.h"
#include "io.h"
#include "sys/alt_alarm.h"


/******************************************************************
*  Function: vga_init_no_interrupt
*
*  Purpose: Initializes the VGA controller for the Lancelot VGA
*           daughterboard.  Gives the controller the base address
*           of the frame buffer, the screen size, and then enables
*           the controller.  Interrupts are NOT enabled.
*
******************************************************************/
//int vga_init_no_interrupt (int *controller_base, vga_frame_buffer_struct *buffer_base)
//{
//
//  volatile int *DMA = controller_base;
//
//  *DMA = (int)buffer_base;  // base address of frame buffer
//  *(DMA+1) = (((int)(FRAME_WIDTH << 16)) | ((int)(FRAME_HEIGHT)));  // image size
//  *(DMA+2) = 0x00000001; // start driver with no interrupt
//
//  return (0);
//}

//void vga_scroll_string_quit(vga_text_scroll_struct* scroll)
//{
//  free(scroll->string);
//  free(scroll);
//}

//vga_text_scroll_struct* vga_scroll_string_init(int hbegin, int vbegin, int hend, int f_color, int b_color, char* font, int ms_delay, char *string)
//{
//  vga_text_scroll_struct* scroll;
//  scroll = malloc(sizeof (vga_text_scroll_struct));
//  
//  scroll->hbegin = hbegin;
//  scroll->vbegin = vbegin;
//  scroll->hend = hend;
//  scroll->f_color = f_color;
//  scroll->b_color = b_color;
//  scroll->string = malloc(strlen(string)+2);
//  strcpy(scroll->string, string);
//  scroll->font = font;
//  scroll->ms_delay = ms_delay;
//  scroll->ticks_at_last_move = alt_nticks();
//  scroll->text_scroll_index = 0;
//  scroll->text_scroll_started = 0;
//  scroll->window_width = scroll->hend - scroll->hbegin;
//  scroll->length_of_string = strlen(string);
//  scroll->string_points = scroll->length_of_string * 8;
//  scroll->scroll_points = (scroll->window_width + scroll->string_points);
// 
//  return(scroll);
//  
//}


//int vga_scroll_string(vga_text_scroll_struct* scroll, vga_frame_buffer_struct* vga_frame_buffer)
//{
//
//  int x_start, x_end, x_index, string_x_index, string_char_index, char_row, char_column;
//  char character, column_mask;
//  char* font_char_ptr;
//  char pixels_to_move_by = 1;
//
//  // If it's time to move the scroll..
//  if (alt_nticks() >= (scroll->ticks_at_last_move + ((alt_ticks_per_second() * (scroll->ms_delay)) / 1000))) {
//    scroll->ticks_at_last_move = alt_nticks();
//    
//    // Track where we are in the scroll.
//    if(scroll->text_scroll_started == 0) {
//      scroll->text_scroll_index = 0;
//      scroll->text_scroll_started = 1;
//    } else if(scroll->text_scroll_index >= scroll->scroll_points)  {
//      scroll->text_scroll_started = 0;
//    } else {
//      scroll->text_scroll_index += pixels_to_move_by;
//    }
//    
//    //Find out where we start
//    if (scroll->text_scroll_index < scroll->window_width) {
//      x_start = scroll->hbegin + scroll->window_width - scroll->text_scroll_index;
//    } else {
//      x_start = scroll->hbegin;
//    }
//    //Find out where we end
//    if (scroll->string_points > scroll->text_scroll_index) {
//      x_end = scroll->hend;
//    } else {
//      x_end = (scroll->hend - scroll->text_scroll_index + scroll->string_points);
//    }
//
//    // Write the string segment a column (x) at a time
//    for(x_index = x_start; x_index < x_end; x_index++) {
//      // Find the x index we're at within the string
//      // If first part of string hasnt yet reached left side of scroll window
//      if (scroll->text_scroll_index < scroll->window_width) {
//        string_x_index = (x_index - x_start);
//      } else {
//        string_x_index = scroll->text_scroll_index - scroll->window_width + x_index - x_start;
//      }
//      //Find the character we're supposed to be writing
//      string_char_index = (string_x_index / 8);
//      character = scroll->string[string_char_index];
//      char_column = (string_x_index % 8);
//      column_mask = (((unsigned int)0x80) >> char_column);
//      font_char_ptr = (scroll->font + ((character - 0x20) * FONT_10PT_ROW));
//      //We have all the data now, so let's write a column
//      for(char_row = 0; char_row < 11; char_row++) {
//        // If the font table says this pixel is on, then set it to the foreground color
//        if (*(font_char_ptr + char_row) & column_mask) {
//          vga_set_pixel(x_index, scroll->vbegin + char_row, scroll->f_color, vga_frame_buffer); 
//        // Otherwise, set it to the background color.
//        } else {
//          vga_set_pixel(x_index, scroll->vbegin + char_row, scroll->b_color, vga_frame_buffer); //background color
//        }
//      }
//    }
//    // Erase the leftover column (x) of the last string we wrote.
//    vga_draw_line(x_end, scroll->vbegin, x_end, scroll->vbegin + 10, 1, scroll->b_color, vga_frame_buffer);
//    // Log what time we moved the scroll.
//  }
//  return(0);
//}


/******************************************************************
*  Function: vga_move_block_8bit
*
*  Purpose: Moves a block around an 8 bit framebuffer, backfilling
*           with the backfill_color parameter.
* 
*           [AC] rewritten to cope with moves in any direction
* 
*           This is not BitBLT!!!
*
******************************************************************/
int vga_move_block_8bit(int xbegin, int ybegin, int xend, int yend, int x_distance, int y_distance, unsigned char backfill_index, unsigned char* frameBuffer)
{
  // Set up source and destination pointers to bytes in frame buffer
  alt_u8* src_ptr = ((alt_u8*) frameBuffer) + ybegin*SCREENWIDTH + xbegin;
  alt_u8* dest_ptr = ((alt_u8*) frameBuffer) + ybegin*SCREENWIDTH +y_distance*SCREENWIDTH + xbegin + x_distance;
  int move_lines = abs(yend-ybegin)+1;
  int move_bytes = abs(xend - xbegin)+1;
  int i;
  
  if ((y_distance >= abs(yend - ybegin)) || (x_distance >= abs(xend - xbegin))) {
    // No overlap between source and destination areas
    // Use memcpy per line
    for (i = 0; i < move_lines; i++) {
      memcpy(dest_ptr, src_ptr, move_bytes);
      src_ptr = src_ptr + SCREENWIDTH;
      dest_ptr = dest_ptr + SCREENWIDTH;
    }
    // paint block to backfill
    vga_paint_block_8bit(xbegin, ybegin, xend, yend, backfill_index, frameBuffer);
    
  } else if (y_distance == 0) {
    // moving horizontally
    // use memmove to copy lines and cope with overlap
    for (i = 0; i < move_lines; i++) {
      memmove(dest_ptr, src_ptr, move_bytes);
      src_ptr = src_ptr + SCREENWIDTH;
      dest_ptr = dest_ptr + SCREENWIDTH;
    }
    // paint block to backfill
    if (x_distance < 0) {
      vga_paint_block_8bit(xend + x_distance, ybegin, xend, yend, backfill_index, frameBuffer);
    } else {
      vga_paint_block_8bit(xbegin, ybegin, xbegin + x_distance, yend, backfill_index, frameBuffer);
    }
    
  } else {
    // overlap
    // use memcpy per line
    if (y_distance < 0) {
      for (i = 0; i < move_lines; i++) {
        memcpy(dest_ptr, src_ptr, move_bytes);
        src_ptr = src_ptr + SCREENWIDTH;
        dest_ptr = dest_ptr + SCREENWIDTH;
      }
    } else {
      src_ptr = src_ptr + SCREENWIDTH*(move_lines-1);
      dest_ptr = dest_ptr + SCREENWIDTH*(move_lines-1);
      for (i = move_lines; i > 0; i--) {
        memcpy(dest_ptr, src_ptr, move_bytes);
        src_ptr = src_ptr - SCREENWIDTH;
        dest_ptr = dest_ptr - SCREENWIDTH;
      }
    }
    // paint block to backfill
    if (y_distance < 0) {
      if (x_distance < 0) {
        vga_paint_block_8bit(xend + x_distance, ybegin, xend, yend + y_distance, backfill_index, frameBuffer);
      } else {
        vga_paint_block_8bit(xbegin, ybegin, xbegin + x_distance, yend + y_distance, backfill_index, frameBuffer);
      }
      vga_paint_block_8bit(xbegin, yend + y_distance + 1, xend, yend, backfill_index, frameBuffer);
    } else {
      vga_paint_block_8bit(xbegin, ybegin, xend, ybegin + y_distance - 1, backfill_index, frameBuffer);
      if (x_distance < 0) {
        vga_paint_block_8bit(xend + x_distance, ybegin + y_distance, xend, yend, backfill_index, frameBuffer);
      } else {
        vga_paint_block_8bit(xbegin, ybegin + y_distance, xbegin + x_distance, yend, backfill_index, frameBuffer);
      }
    }
  }
  return (0);
}


/******************************************************************
*  Function: vga_print_string
*
*  Purpose: Prints a string to the specified location of the screen
*           using the specified font and color.
*           Calls vga_print_char
*
******************************************************************/
//int vga_print_string(int horiz_offset, int vert_offset, int color, char *font, vga_frame_buffer_struct* vga_frame_buffer, char string[])
//{
//  int i = 0;
//  int original_horiz_offset;
//
//  original_horiz_offset = horiz_offset;
//
//  // Print until we hit the '\0' char.
//  while (string[i]) {
//    //Handle newline char here.
//    if (string[i] == '\n') {
//      horiz_offset = original_horiz_offset;
//      vert_offset += 12;
//      i++;
//      continue;
//    }
//    // Lay down that character and increment our offsets.
//    vga_print_char (horiz_offset, vert_offset, color, string[i], font, vga_frame_buffer);
//    i++;
//    horiz_offset += 8;
//  }
//  return (0);
//}


/******************************************************************
*  Function: vga_print_string_8bit
*
*  Purpose: Prints a string to the specified location of an 8bit
*           frame buffer using the specified font and pallette
*           index.
*
******************************************************************/
int vga_print_string_8bit(int horiz_offset, int vert_offset, unsigned char index, char *font, int size, unsigned char* frameBuffer, char string[])
{
  int i = 0;
  int original_horiz_offset;

  original_horiz_offset = horiz_offset;

  // Print until we hit the '\0' char.
  while (string[i]) {
    //Handle newline char here.
    if (string[i] == '\n') {
      horiz_offset = original_horiz_offset;
      vert_offset += size*12;
      i++;
      continue;
    }
    // Lay down that character and increment our offsets.
    vga_print_char_8bit(horiz_offset, vert_offset, index, string[i], font, size, frameBuffer);
    i++;
    horiz_offset += size*8;
  }
  return (0);
}


/******************************************************************
*  Function: vga_draw_box
*
*  Purpose: Draws a box on the screen with the specified corner
*           points.  The fill parameter tells the function whether
*           or not to fill in the box.  1 = fill, 0 = do not fill.
* 
*  [AC] now uses modified vga_paint_block to handle odd/even pixel
*       issues. Uses optimized line draw routines for horizontal
*       and vertical lines of open box.
*
******************************************************************/
int vga_draw_box (int Hstart, int Vstart, int Hend, int Vend, short color, int fill, short* frameBuffer)
{
  // Calculate corner pixels
  int ulp = Vstart*SCREENWIDTH+Hstart;
  int urp = Vstart*SCREENWIDTH+Hend;
  int llp = Vend*SCREENWIDTH+Hstart;
  int lrp = Vend*SCREENWIDTH+Hend;
  // If we want to fill in our box
  if (fill) {
    vga_paint_block (Hstart, Vstart, Hend, Vend, color, frameBuffer);
  } else {
  // If we're not filling in the box, just draw four lines.
    vga_horiz_line(ulp, urp, color, frameBuffer);
    vga_horiz_line(llp, lrp, color, frameBuffer);
    vga_vert_line(ulp, llp, color, frameBuffer);
    vga_vert_line(urp, lrp, color, frameBuffer);
  }
  return (0);
}

/******************************************************************
*  Function: vga_draw_box_8bit
*
*  Purpose: Draws a box in an 8 bit framebuffer with the specified
*           corner points.  The fill parameter tells the function
*           whether or not to fill in the box.  
*               1 = fill, 0 = do not fill.
*           index is index into colour pallette
* 
*           Note: x and y start must be less than x and y end
* 
*           [AC] Added width parameter which only applies to
*                unfilled boxes
* 
******************************************************************/
int vga_draw_box_8bit (int Hstart, int Vstart, int Hend, int Vend, alt_32 width, unsigned char index, int fill, unsigned char* frameBuffer)
{
  // Calculate corner pixels
  int ulp = Vstart*SCREENWIDTH+Hstart;
  int urp = Vstart*SCREENWIDTH+Hend;
  int llp = Vend*SCREENWIDTH+Hstart;
  int lrp = Vend*SCREENWIDTH+Hend;
  // If we want to fill in our box
  if (fill) {
    vga_paint_block_8bit (Hstart, Vstart, Hend, Vend, index, frameBuffer);
  } else {
  // If we're not filling in the box, just draw four lines.
    vga_horiz_line_8bit(ulp, urp, width, index, frameBuffer);
    vga_horiz_line_8bit(llp, lrp, width, index, frameBuffer);
    vga_vert_line_8bit(ulp, llp, width, index, frameBuffer);
    vga_vert_line_8bit(urp, lrp, width, index, frameBuffer);
  }
  return (0);
}


/******************************************************************
*  Function: vga_print_char
*
*  Purpose: Prints a character to the specified location of the
*           screen using the specified font and color.
*
******************************************************************/
//int vga_print_char (int horiz_offset, int vert_offset, int color, char character, char *font, vga_frame_buffer_struct* vga_frame_buffer)
//{
//
//  int i, j;
//  
//  char temp_char, char_row;
//
//  // Convert the ASCII value to an array offset
//  temp_char = (character - 0x20);
//
//  //Each character is 8 pixels wide and 11 tall.
//  for(i = 0; i < 11; i++) {
//      char_row = *(font + (temp_char * FONT_10PT_ROW) + i);
//    for (j = 0; j < 8; j++) {
//      //If the font table says the pixel in this location is on for this character, then set it.
//      if (char_row & (((unsigned char)0x80) >> j)) {
//        vga_set_pixel((horiz_offset + j), (vert_offset + i), color, vga_frame_buffer); // plot the pixel
//      }
//    }
//  }
//  return(0);
//}


/******************************************************************
*  Function: vga_print_char_8bit
*
*  Purpose: Prints a character to the specified location of the
*           screen using the specified font and color.
*
******************************************************************/
int vga_print_char_8bit (int horiz_offset, int vert_offset, unsigned char index, char character, char *font, int size, unsigned char* frameBuffer)
{
  int i, j;
  char temp_char, char_row;

  // Convert the ASCII value to an array offset
  temp_char = (character - 0x20);

  //Each character is 8 pixels wide and 11 tall, modified by integer size factor.
  for(i = 0; i < size*11; i++) {
    char_row = *(font + (temp_char * FONT_10PT_ROW) + i/size);
    for (j = 0; j < size*8; j++) {
      //If the font table says the pixel in this location is on for this character, then set it.
      if (char_row & (((unsigned char)0x80) >> j/size)) {
        vga_set_pixel_8bit((horiz_offset + j), (vert_offset + i), index, frameBuffer); // plot the pixel
      }
    }
  }
  return(0);
}


/******************************************************************
*  Function: vga_get_pixel
*
*  Purpose: Reads the color of the pixel at the given coordinates
*
******************************************************************/
//short vga_get_pixel(int horiz, int vert, vga_frame_buffer_struct* vga_frame_buffer)
//{
//  // get 1 pixel (16 bits).  Offset by two if the address is odd.
//  return(IORD_16DIRECT((&(vga_frame_buffer->vga_frame[vert][horiz/2])), (horiz & 0x1)*2));
//
//}


/******************************************************************
*  Function: vga_paint_block
*
*  Purpose: Paints a block of the screen the specified color.
*           Note: works with two pixels at a time to maintain high
*           bandwidth.  Therefore, corner points must be even
*           numbered coordinates.  Use vga_draw_solid_box for
*           areas with odd-numbered corner points.
*           The color parameter must contain two pixel's worth
*           (32 bits).
* 
* [AC] Modified do it works with odd or even corner points
*
******************************************************************/
void vga_paint_block (int Hstart, int Vstart, int Hend, int Vend, short color, short* frameBuffer) {
  int line, pixel, last_pixel;
  for (line = Vstart; line <= Vend; line++) {
    pixel = line*SCREENWIDTH + Hstart;
    last_pixel = pixel + Hend - Hstart;
    // Draw as much of line as we can two pixels at a time
    vga_horiz_line(pixel, last_pixel, color, frameBuffer);
  }
}

/******************************************************************
*  Function: vga_paint_block_8bit
*
*  Purpose: Paints a block in an 8 bit frame buffer the specified
*           pallette index.
*
******************************************************************/
void vga_paint_block_8bit (int Hstart, int Vstart, int Hend, int Vend, unsigned char index, unsigned char* frameBuffer) {
  int line, pixel, last_pixel;
  for (line = Vstart; line <= Vend; line++) {
    pixel = line*SCREENWIDTH + Hstart;
    last_pixel = pixel + Hend - Hstart;
    // Draw as much of line as we can two pixels at a time
    vga_horiz_line_8bit(pixel, last_pixel, 1, index, frameBuffer);
  }
}


/******************************************************************
*  Function: vga_horiz_line
*
*  Purpose: Draws a horizontal line between and including two
*           pixels. Calls an inline function to handle as much of
*           the line as possible two pixels at a time
* 
******************************************************************/
void vga_horiz_line (int start, int end, short color, short* frameBuffer) {
  vga_sort_line(&start, &end);
  // Handle odd pixel at start of line
  if (start & 1) {
    frameBuffer[start++] = color;
  }
  // Handle even pixel at end of line
  if ((end & 1) == 0) {
    frameBuffer[end--] = color;
  }
  // Draw middle of line two pixels at a time
  if (start < end) {
    draw_hline(start, end, color, frameBuffer);
  }
}

/******************************************************************
*  Function: vga_horiz_line_8bit
*
*  Purpose: Draws a horizontal line into an 8 bit frame buffer 
*           between and including two pixels. Calls an inline
*           function to handle as much of the line as possible four
*           pixels at a time.
* 
******************************************************************/
#ifdef GRAPHICS_2D_0_BASE

  void vga_horiz_line_8bit (int start, int end, alt_32 width, unsigned char index, unsigned char* frameBuffer) {
//    alt_32 i;
    alt_32 xstart, xend, ystart, yend;
    vga_sort_line(&start, &end);
  
    alt_u8* ptr = (alt_u8*)(((alt_u32)frameBuffer) & 0x7fffffff);

    ystart = xstart = (alt_u32)ptr + start - (width/2) * SCREENWIDTH;
    xend = (alt_u32)ptr + end - (width/2) * SCREENWIDTH;
    yend = (alt_u32)ptr + end + ((width - width/2) - 1) * SCREENWIDTH;

    while (IORD(GRAPHICS_2D_0_BASE, 0x0) > 0) {
    }
    IOWR(GRAPHICS_2D_0_BASE, (0x10>>2), index);
    IOWR(GRAPHICS_2D_0_BASE, (0x20>>2), xstart);
    IOWR(GRAPHICS_2D_0_BASE, (0x24>>2), xend);
    IOWR(GRAPHICS_2D_0_BASE, (0x28>>2), ystart);
    IOWR(GRAPHICS_2D_0_BASE, (0x2c>>2), yend);
    IOWR(GRAPHICS_2D_0_BASE, 0x0, 0x2);


//    for (i = -width/2; i < (width - width/2); i++) {
//      startex = start + i*SCREENWIDTH;
//      endex = end + i*SCREENWIDTH;
//      while (IORD(GRAPHICS_2D_0_BASE, 0x0) > 0) {
//      }
//      IOWR(GRAPHICS_2D_0_BASE, (0x20>>2), startex);
//      IOWR(GRAPHICS_2D_0_BASE, (0x24>>2), endex);
//      IOWR(GRAPHICS_2D_0_BASE, 0x0, 0x1);
//    }
  }

#else

  void vga_horiz_line_8bit (int start, int end, alt_32 width, unsigned char index, unsigned char* frameBuffer) {
    alt_32 i;
    alt_32 startex, endex;
    vga_sort_line(&start, &end);
  
    for (i = -width/2; i < (width - width/2); i++) {
      // Handle pixels at start of line
      startex = start + i*SCREENWIDTH;
      endex = end + i*SCREENWIDTH;
      switch (startex & 3) {
        case 1:
          frameBuffer[startex++] = index;
        case 2:
          frameBuffer[startex++] = index;
        case 3:
          frameBuffer[startex++] = index;
          break;
        default:
          break;
      }
      // Handle pixels at end of line
      switch (endex & 3) {
        case 2:
          frameBuffer[endex--] = index;
        case 1:
          frameBuffer[endex--] = index;
        case 0:
          frameBuffer[endex--] = index;
          break;
        default:
          break;
      }
      // Draw middle of line two pixels at a time
      if (startex < endex) {
        draw_hline_8bit(startex, endex, index, frameBuffer);
      }
    }
  }

#endif

/******************************************************************
*  Function: vga_vert_line
*
*  Purpose: Draws a vertical line between and including two
*           pixels.
* 
*           NOTE: start must be less than end
* 
******************************************************************/
void vga_vert_line (int start, int end, short color, short* frameBuffer) {
  vga_sort_line(&start, &end);
  int i;
  for (i = start; i <= end; i += SCREENWIDTH)
    frameBuffer[i] = color;
}

/******************************************************************
*  Function: vga_merge_colors
*
*  Purpose: Takes 5-bit color values for each red, green, and blue
*           and merges them into one 16-bit color value.
*
******************************************************************/
int vga_merge_colors(int red, int green, int blue)
{
  // Green actually has 6-bits, but we'll make it's LSB 1 to be consistent.
  return ((blue) | (((green << 1) | 0x1) << 5) | (red << 11));
}


/******************************************************************
*  Function: vga_draw_line
*
*  Purpose: Draws a line between two end points using
*           Bresenham's line drawing algorithm.
*           width parameter is not used.  
*           It is reserved for future use.
*
******************************************************************/
void vga_draw_line(int horiz_start, int vert_start, int horiz_end, int vert_end, int width, int color, short* frameBuffer)
{
  // Find the vertical and horizontal distance between the two points
  int horiz_delta = abs(horiz_end-horiz_start);
  int vert_delta = abs(vert_end-vert_start);

  // Find out what direction we are going
  int horiz_incr, vert_incr;
  if (horiz_start > horiz_end) { horiz_incr=-1; } else { horiz_incr=1; }
  if (vert_start > vert_end) { vert_incr=-1; } else { vert_incr=1; }

  // Find out which axis is always incremented when drawing the line
  // If it's the horizontal axis
  if (horiz_delta >= vert_delta) {
    int dPr   = vert_delta<<1;
    int dPru  = dPr - (horiz_delta<<1);
    int P     = dPr - horiz_delta;

    // Process the line, one horizontal point at at time
    for (; horiz_delta >= 0; horiz_delta--) {
      // plot the pixel
      vga_set_pixel(horiz_start, vert_start, color, frameBuffer);
      // If we're moving both up and right
      if (P > 0) {
        horiz_start+=horiz_incr;
        vert_start+=vert_incr;
        P+=dPru;
      } else {
        horiz_start+=horiz_incr;
        P+=dPr;
      }
    }
  // If it's the vertical axis
  } else {
    int dPr   = horiz_delta<<1;
    int dPru  = dPr - (vert_delta<<1);
    int P     = dPr - vert_delta;

    // Process the line, one vertical point at at time
    for (; vert_delta>=0; vert_delta--) {
      // plot the pixel
      vga_set_pixel(horiz_start, vert_start, color, frameBuffer);
      // If we're moving both up and right
      if (P > 0) {
        horiz_start+=horiz_incr;
        vert_start+=vert_incr;
        P+=dPru;
      } else {
        vert_start+=vert_incr;
        P+=dPr;
      }
    }
  }
}


/******************************************************************
*  Function: vga_draw_line_8bit
*
*  Purpose: Draws a line between two end points using
*           Bresenham's line drawing algorithm.
*           width parameter is not used.  
*           It is reserved for future use.
* 
*           [AC] optimized the loop code for a 75+% (measured)
*           inprovement in drawing speed.
*
******************************************************************/
void vga_draw_line_8bit(int horiz_start, int vert_start, int horiz_end, int vert_end, int width, unsigned char index, unsigned char* frameBuffer)
{
  alt_32 tmp, i;
  alt_32 start_line, end_line;
  if (vert_start == vert_end) {
    // If line is horizontal, use optimized routine
    start_line = vert_start * SCREENWIDTH;
    vga_horiz_line_8bit (start_line + horiz_start, start_line + horiz_end, width, index, frameBuffer);
  } else if (horiz_start == horiz_end) {
    // If line is vertial, use optimized routine
    start_line = vert_start * SCREENWIDTH;
    end_line = vert_end * SCREENWIDTH;
    vga_vert_line_8bit (start_line + horiz_start, end_line + horiz_end, width, index, frameBuffer);
  } else {
  
    // Find the vertical and horizontal distance between the two points
    int horiz_delta = abs(horiz_end-horiz_start);
    int vert_delta = abs(vert_end-vert_start);
  
    // Find out what direction we are going
    int horiz_incr, vert_incr;
    if (horiz_start > horiz_end) {horiz_incr=-1;} else {horiz_incr=1;}
    if (vert_start > vert_end) {vert_incr=-1;} else {vert_incr=1;}
  
    // Find out which axis is always incremented when drawing the line
    // If it's the horizontal axis
    if (horiz_delta >= vert_delta) {
      int dPr   = vert_delta<<1;
      int dPru  = dPr - (horiz_delta<<1);
      int P     = dPr - horiz_delta;
  
      // [AC] 
      if (horiz_incr > 0) {
        tmp = horiz_start;
        horiz_start = horiz_end;
        horiz_end = tmp;
        tmp = vert_start;
        vert_start = vert_end;
        vert_end = tmp;
        horiz_incr = -1;
        if (vert_start > vert_end) { vert_incr=-1; } else { vert_incr=1; }
      }
      
      // Process the line, one horizontal point at at time
      for (; horiz_start >= horiz_end; horiz_start--) {
        // plot the pixel
        for (i = -width/2; i < (width - width/2); i++) {
          vga_set_pixel_8bit(horiz_start, vert_start + i, index, frameBuffer);
        }
        // If we're moving both up and right
        if (P > 0) {
          vert_start+=vert_incr;
          P+=dPru;
        } else {
          P+=dPr;
        }
      }
    // If it's the vertical axis
    } else {
      int dPr   = horiz_delta<<1;
      int dPru  = dPr - (vert_delta<<1);
      int P     = dPr - vert_delta;
  
      // [AC] 
      if (vert_incr > 0) {
        tmp = horiz_start;
        horiz_start = horiz_end;
        horiz_end = tmp;
        tmp = vert_start;
        vert_start = vert_end;
        vert_end = tmp;
        vert_incr = -1;
        if (horiz_start > horiz_end) { horiz_incr=-1; } else { horiz_incr=1; }
      }
      
      // Process the line, one vertical point at at time
      for (; vert_start>=vert_end; vert_start--) {
        // plot the pixel
        for (i = -width/2; i < (width - width/2); i++) {
          vga_set_pixel_8bit(horiz_start + i, vert_start, index, frameBuffer);
        }
        // If we're moving both up and right
        if (P > 0) {
          horiz_start+=horiz_incr;
          P+=dPru;
        } else {
          P+=dPr;
        }
      }
    }
  }
}


/******************************************************************
*  Function: vga_draw_polyline_8bit
*
*  Purpose: Draws a polyline between points
*           width parameter is not used.  
*           It is reserved for future use.
*
******************************************************************/
void vga_draw_polyline_8bit(point points[], int num_points, int width, unsigned char index, unsigned char* frameBuffer)
{
  int i;
  for (i=0; i<num_points-1; i++) {
    vga_draw_line_8bit(points[i].x, points[i].y, points[i+1].x, points[i+1].y, width, index, frameBuffer);
  }
}


/******************************************************************
*  Function: vga_draw_polygon_8bit
*
*  Purpose: Draws a closed polygon between points width parameter
*           is not used.  
*           It is reserved for future use.
* 
*           Handles only convex polygons.
*
******************************************************************/
void vga_draw_polygon_8bit(point points[], alt_32 num_points, alt_32 width, alt_u8 index, alt_32 fill, alt_u8* frameBuffer)
{
  if (fill) {
    edge edges[MAX_VERTICES-1];
    edge active_edges[2];
    alt_32 i, j, end_point, num_edges = 0, line, maxy = 0, current_edge;
    alt_32 tmpsx, tmpsy, tmpex, tmpey;
    alt_32 x[2], xint[2];
    // generate edges
    for (i=0; i<num_points; i++) {
      if (i == num_points - 1) {
        end_point = 0;
      } else {
        end_point = i + 1;
      }
      if (points[i].y == points[end_point].y) {
        // Discard horizontal edges
        continue;
      }
      // Edges start from minimum y value
      if (points[i].y < points[end_point].y) {
        (edges[num_edges].start).x = points[i].x;
        (edges[num_edges].start).y = points[i].y;
        (edges[num_edges].end).x = points[end_point].x;
        (edges[num_edges].end).y = points[end_point].y;
      } else {
        (edges[num_edges].start).x = points[end_point].x;
        (edges[num_edges].start).y = points[end_point].y;
        (edges[num_edges].end).x = points[i].x;
        (edges[num_edges].end).y = points[i].y;
      }
      num_edges++;
    }
    
    // Sort edge array by y then x
    for (i = 1; i < num_edges; i++) {
      for (j = 0; j < num_edges - i; j++) {
        if ((edges[j].start).y > (edges[j+1].start).y) {
//          vga_swap_edges(&edges[j], &edges[j+1]);
          tmpsx = (edges[j].start).x;
          tmpsy = (edges[j].start).y;
          tmpex = (edges[j].end).x;
          tmpey = (edges[j].end).y;
          (edges[j].start).x = (edges[j+1].start).x;
          (edges[j].start).y = (edges[j+1].start).y;
          (edges[j].end).x = (edges[j+1].end).x;
          (edges[j].end).y = (edges[j+1].end).y;
          (edges[j+1].start).x = tmpsx;
          (edges[j+1].start).y = tmpsy;
          (edges[j+1].end).x = tmpex;
          (edges[j+1].end).y = tmpey;
        } else if ((edges[j].start).y == (edges[j+1].start).y) {
          if ((edges[j].start).x > (edges[j+1].start).x) {
//            vga_swap_edges(&edges[j], &edges[j+1]);
            tmpsx = (edges[j].start).x;
            tmpsy = (edges[j].start).y;
            tmpex = (edges[j].end).x;
            tmpey = (edges[j].end).y;
            (edges[j].start).x = (edges[j+1].start).x;
            (edges[j].start).y = (edges[j+1].start).y;
            (edges[j].end).x = (edges[j+1].end).x;
            (edges[j].end).y = (edges[j+1].end).y;
            (edges[j+1].start).x = tmpsx;
            (edges[j+1].start).y = tmpsy;
            (edges[j+1].end).x = tmpex;
            (edges[j+1].end).y = tmpey;
          }
        }
      }
    }
    
    // Calculate 1/slope of each edge x 4096 to avoid need for floating point
    // and find max y
    for (i = 0; i < num_edges; i++) {
      edges[i].slope = (((edges[i].end).x - (edges[i].start).x)*4096)/((edges[i].end).y - (edges[i].start).y);
      if ((edges[i].start).y > maxy) maxy = (edges[i].start).y;
      if ((edges[i].end).y > maxy) maxy = (edges[i].end).y;
    }
    
    // Get first two active edges
    for (i = 0; i < 2; i++) {
      (active_edges[i].start).x = (edges[i].start).x;
      (active_edges[i].start).y = (edges[i].start).y;
      (active_edges[i].end).x = (edges[i].end).x;
      (active_edges[i].end).y = (edges[i].end).y;
      active_edges[i].slope = edges[i].slope;
    }
    current_edge = 2;
    
    x[0] = 4096*(active_edges[0].start).x;
    x[1] = 4096*(active_edges[1].start).x;
    xint[0] = x[0]/4096;
    xint[1] = x[1]/4096;
    // Start on scan line of minimum y, don't plot last line (max y)
    for (line = (edges[0].start).y; line < maxy;) {
      // Draw a horizontal line
      vga_horiz_line_8bit (line*SCREENWIDTH + xint[0], line*SCREENWIDTH + xint[1], 1, index, frameBuffer);

      // update x values
      x[0] = x[0] + active_edges[0].slope;
      x[1] = x[1] + active_edges[1].slope;
      xint[0] = x[0]/4096;
      xint[1] = x[1]/4096;

      // Replace edges that we're finished with
      line++;
      for (i = 0; i < 2; i++) {
        if (((active_edges[i].end).y == line) && (current_edge < num_edges)) {
          (active_edges[i].start).x = (edges[current_edge].start).x;
          (active_edges[i].start).y = (edges[current_edge].start).y;
          (active_edges[i].end).x = (edges[current_edge].end).x;
          (active_edges[i].end).y = (edges[current_edge].end).y;
          active_edges[i].slope = edges[current_edge].slope;
          current_edge++;
          x[i] = 4096*(active_edges[i].start).x;
          xint[i] = x[i]/4096;
        }
      }
      
    }
  } else {
    vga_draw_polyline_8bit(points, num_points, width, index, frameBuffer);
    vga_draw_line_8bit(points[num_points-1].x, points[num_points-1].y, points[0].x, points[0].y, width, index, frameBuffer);
  }
}


void vga_swap_edges(edge *edge1, edge *edge2) {
  alt_32 tmpsx, tmpsy, tmpex, tmpey;
  tmpsx = ((*edge1).start).x;
  tmpsy = ((*edge1).start).y;
  tmpex = ((*edge1).end).x;
  tmpey = ((*edge1).end).y;
  ((*edge1).start).x = ((*edge2).start).x;
  ((*edge1).start).y = ((*edge2).start).y;
  ((*edge1).end).x = ((*edge2).end).x;
  ((*edge1).end).y = ((*edge2).end).y;
  ((*edge2).start).x = tmpsx;
  ((*edge2).start).y = tmpsy;
  ((*edge2).end).x = tmpex;
  ((*edge2).end).y = tmpey;
}

/******************************************************************
*  Function: vga_draw_circle_8bit
*
*  Purpose: Draws a circle on the screen with the specified center
*  and radius.  Draws symetric circles only.  The fill parameter
*  tells the function whether or not to fill in the box.  1 = fill,
*  0 = do not fill.
* 
******************************************************************/
int vga_draw_circle_8bit(int Hcenter, int Vcenter, int radius, int width, unsigned char index, int fill, unsigned char* frameBuffer)
{
  int x, y, p;
  x = 0;
  y = radius+width/2;
  p = (5 - y*4)/4;

  // Start the circle with the top, bottom, left, and right pixels.
  vga_circle_points_8bit(Hcenter, Vcenter, x, y, index, fill, frameBuffer);

  // Now start moving out from those points until the lines meet
  while (x < y) {
    x++;
    if (p < 0) {
      p += 2*x+1;
    } else {
      y--;
      p += 2*(x-y)+1;
    }
    vga_circle_points_8bit(Hcenter, Vcenter, x, y, index, fill, frameBuffer);
  }
  return (0);
}

/******************************************************************
*  Function: vga_circle_points_8bit
*
*  Purpose: Called by vga_draw_circle() to plot the actual points
*  of the circle.  Draws horizontal lines to fill.
*
******************************************************************/
void vga_circle_points_8bit(int cx, int cy, int x, int y, unsigned char index, int fill, unsigned char* frameBuffer)
{
  // If we're directly above, below, left and right of center (0 degrees), plot those 4 pixels
  if (x == 0) {
    vga_set_pixel_8bit(cx, cy + y, index, frameBuffer);
    vga_set_pixel_8bit(cx, cy - y, index, frameBuffer);
    if(fill) {
      vga_horiz_line_8bit(cy*SCREENWIDTH + cx - y, cy*SCREENWIDTH + cx + y, 1, index, frameBuffer);
    } else {
        vga_set_pixel_8bit(cx + y, cy, index, frameBuffer);
        vga_set_pixel_8bit(cx - y, cy, index, frameBuffer);
    }

  } else
  // If we've reached the 45 degree points (x=y), plot those 4 pixels
  if (x == y) {
    if(fill) {
      vga_horiz_line_8bit((cy + y)*SCREENWIDTH + cx - x, (cy + y)*SCREENWIDTH + cx + x, 1, index, frameBuffer);
      vga_horiz_line_8bit((cy - y)*SCREENWIDTH + cx - x, (cy - y)*SCREENWIDTH + cx + x, 1, index, frameBuffer);
    } else {
        vga_set_pixel_8bit(cx + x, cy + y, index, frameBuffer);
        vga_set_pixel_8bit(cx - x, cy + y, index, frameBuffer);
        vga_set_pixel_8bit(cx + x, cy - y, index, frameBuffer);
        vga_set_pixel_8bit(cx - x, cy - y, index, frameBuffer);
    }
  } else
  // If we're between 0 and 45 degrees plot 8 pixels.
  if (x < y) {
    if(fill) {
      vga_horiz_line_8bit((cy + y)*SCREENWIDTH + cx - x, (cy + y)*SCREENWIDTH + cx + x, 1, index, frameBuffer);
      vga_horiz_line_8bit((cy + x)*SCREENWIDTH + cx - y, (cy + x)*SCREENWIDTH + cx + y, 1, index, frameBuffer);
      vga_horiz_line_8bit((cy - x)*SCREENWIDTH + cx - y, (cy - x)*SCREENWIDTH + cx + y, 1, index, frameBuffer);
      vga_horiz_line_8bit((cy - y)*SCREENWIDTH + cx - x, (cy - y)*SCREENWIDTH + cx + x, 1, index, frameBuffer);
    } else {
        vga_set_pixel_8bit(cx + x, cy + y, index, frameBuffer);
        vga_set_pixel_8bit(cx - x, cy + y, index, frameBuffer);
        vga_set_pixel_8bit(cx + x, cy - y, index, frameBuffer);
        vga_set_pixel_8bit(cx - x, cy - y, index, frameBuffer);
        vga_set_pixel_8bit(cx + y, cy + x, index, frameBuffer);
        vga_set_pixel_8bit(cx - y, cy + x, index, frameBuffer);
        vga_set_pixel_8bit(cx + y, cy - x, index, frameBuffer);
        vga_set_pixel_8bit(cx - y, cy - x, index, frameBuffer);
    }
  }
}


/******************************************************************
*  Function: vga_paint_screen
*
*  Purpose: Paints the whole frame buffer using the specified
*           color.
* 
******************************************************************/
void vga_paint_screen (short colour, short* frameBuffer) {
  vga_paint_block (0, 0, SCREENWIDTH-1, SCREENHEIGHT-1, colour, frameBuffer);
}

/******************************************************************
*  Function: vga_paint_screen_8bit
*
*  Purpose: Paints the whole 8 bit frame buffer using the specified
*           index.
* 
******************************************************************/
void vga_paint_screen_8bit (unsigned char index, unsigned char* frameBuffer) {
  vga_paint_block_8bit (0, 0, SCREENWIDTH-1, SCREENHEIGHT-1, index, frameBuffer);
}


/******************************************************************
*  Data: cour10_font
*
*  Purpose: Data array that represents a 10-point courier font.
*
******************************************************************/
char cour10_font_array[95][11] =

 {{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
 {0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x00, 0x10, 0x00, 0x00},
 {0x28, 0x28, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
 {0x14, 0x14, 0x7E, 0x28, 0x28, 0x28, 0xFC, 0x50, 0x50, 0x00, 0x00},
 {0x10, 0x38, 0x44, 0x40, 0x38, 0x04, 0x44, 0x38, 0x10, 0x00, 0x00},
 {0x40, 0xA2, 0x44, 0x08, 0x10, 0x20, 0x44, 0x8A, 0x04, 0x00, 0x00},
 {0x30, 0x40, 0x40, 0x20, 0x60, 0x92, 0x94, 0x88, 0x76, 0x00, 0x00},
 {0x10, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
 {0x08, 0x10, 0x10, 0x20, 0x20, 0x20, 0x20, 0x20, 0x10, 0x10, 0x08},
 {0x20, 0x10, 0x10, 0x08, 0x08, 0x08, 0x08, 0x08, 0x10, 0x10, 0x20},
 {0x00, 0x00, 0x6C, 0x38, 0xFE, 0x38, 0x6C, 0x00, 0x00, 0x00, 0x00},
 {0x00, 0x10, 0x10, 0x10, 0xFE, 0x10, 0x10, 0x10, 0x00, 0x00, 0x00},
 {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x20, 0x00},
 {0x00, 0x00, 0x00, 0x00, 0xFE, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
 {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00},
 {0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80, 0x00, 0x00, 0x00, 0x00},
 {0x38, 0x44, 0x44, 0x44, 0x44, 0x44, 0x44, 0x44, 0x38, 0x00, 0x00},
 {0x10, 0x70, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x7C, 0x00, 0x00},
 {0x38, 0x44, 0x04, 0x04, 0x08, 0x10, 0x20, 0x40, 0x7C, 0x00, 0x00},
 {0x38, 0x44, 0x04, 0x04, 0x18, 0x04, 0x04, 0x44, 0x38, 0x00, 0x00},
 {0x08, 0x18, 0x18, 0x28, 0x28, 0x48, 0x7C, 0x08, 0x1C, 0x00, 0x00},
 {0x7C, 0x40, 0x40, 0x40, 0x78, 0x04, 0x04, 0x44, 0x38, 0x00, 0x00},
 {0x18, 0x20, 0x40, 0x40, 0x78, 0x44, 0x44, 0x44, 0x38, 0x00, 0x00},
 {0x7C, 0x44, 0x04, 0x08, 0x08, 0x10, 0x10, 0x20, 0x20, 0x00, 0x00},
 {0x38, 0x44, 0x44, 0x44, 0x38, 0x44, 0x44, 0x44, 0x38, 0x00, 0x00},
 {0x38, 0x44, 0x44, 0x44, 0x3C, 0x04, 0x04, 0x08, 0x30, 0x00, 0x00},
 {0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00},
 {0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x10, 0x20, 0x00, 0x00, 0x00},
 {0x00, 0x04, 0x08, 0x10, 0x20, 0x10, 0x08, 0x04, 0x00, 0x00, 0x00},
 {0x00, 0x00, 0x00, 0x7C, 0x00, 0x7C, 0x00, 0x00, 0x00, 0x00, 0x00},
 {0x00, 0x20, 0x10, 0x08, 0x04, 0x08, 0x10, 0x20, 0x00, 0x00, 0x00},
 {0x38, 0x44, 0x04, 0x04, 0x08, 0x10, 0x10, 0x00, 0x10, 0x00, 0x00},
 {0x3C, 0x42, 0x9A, 0xAA, 0xAA, 0xAA, 0x9C, 0x40, 0x38, 0x00, 0x00},
 {0x30, 0x10, 0x10, 0x28, 0x28, 0x44, 0x7C, 0x44, 0xEE, 0x00, 0x00},
 {0xFC, 0x42, 0x42, 0x42, 0x7C, 0x42, 0x42, 0x42, 0xFC, 0x00, 0x00},
 {0x3C, 0x42, 0x80, 0x80, 0x80, 0x80, 0x80, 0x42, 0x3C, 0x00, 0x00},
 {0xF8, 0x44, 0x42, 0x42, 0x42, 0x42, 0x42, 0x44, 0xF8, 0x00, 0x00},
 {0xFE, 0x42, 0x40, 0x48, 0x78, 0x48, 0x40, 0x42, 0xFE, 0x00, 0x00},
 {0xFE, 0x42, 0x40, 0x48, 0x78, 0x48, 0x40, 0x40, 0xF0, 0x00, 0x00},
 {0x3C, 0x42, 0x80, 0x80, 0x80, 0x8E, 0x82, 0x42, 0x3C, 0x00, 0x00},
 {0xEE, 0x44, 0x44, 0x44, 0x7C, 0x44, 0x44, 0x44, 0xEE, 0x00, 0x00},
 {0x7C, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x7C, 0x00, 0x00},
 {0x1E, 0x04, 0x04, 0x04, 0x04, 0x04, 0x44, 0x44, 0x38, 0x00, 0x00},
 {0xE6, 0x44, 0x48, 0x48, 0x50, 0x70, 0x48, 0x44, 0xE6, 0x00, 0x00},
 {0xF8, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x22, 0xFE, 0x00, 0x00},
 {0xC6, 0x44, 0x6C, 0x6C, 0x54, 0x54, 0x44, 0x44, 0xEE, 0x00, 0x00},
 {0xCE, 0x44, 0x64, 0x64, 0x54, 0x4C, 0x4C, 0x44, 0xE4, 0x00, 0x00},
 {0x38, 0x44, 0x82, 0x82, 0x82, 0x82, 0x82, 0x44, 0x38, 0x00, 0x00},
 {0xFC, 0x42, 0x42, 0x42, 0x7C, 0x40, 0x40, 0x40, 0xF0, 0x00, 0x00},
 {0x38, 0x44, 0x82, 0x82, 0x82, 0x82, 0x82, 0x44, 0x38, 0x36, 0x00},
 {0xFC, 0x42, 0x42, 0x42, 0x7C, 0x48, 0x48, 0x44, 0xE6, 0x00, 0x00},
 {0x7C, 0x82, 0x80, 0x80, 0x7C, 0x02, 0x02, 0x82, 0x7C, 0x00, 0x00},
 {0xFE, 0x92, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x38, 0x00, 0x00},
 {0xEE, 0x44, 0x44, 0x44, 0x44, 0x44, 0x44, 0x44, 0x38, 0x00, 0x00},
 {0xEE, 0x44, 0x44, 0x44, 0x28, 0x28, 0x28, 0x10, 0x10, 0x00, 0x00},
 {0xEE, 0x44, 0x44, 0x44, 0x54, 0x54, 0x54, 0x28, 0x28, 0x00, 0x00},
 {0xEE, 0x44, 0x28, 0x28, 0x10, 0x28, 0x28, 0x44, 0xEE, 0x00, 0x00},
 {0xEE, 0x44, 0x44, 0x28, 0x28, 0x10, 0x10, 0x10, 0x38, 0x00, 0x00},
 {0xFE, 0x84, 0x08, 0x08, 0x10, 0x20, 0x20, 0x42, 0xFE, 0x00, 0x00},
 {0x38, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x38, 0x00, 0x00},
 {0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x00, 0x00, 0x00, 0x00},
 {0x1C, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x1C, 0x00, 0x00},
 {0x10, 0x28, 0x44, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
 {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFE, 0x00, 0x00},
 {0x20, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
 {0x00, 0x00, 0x00, 0x78, 0x04, 0x7C, 0x84, 0x84, 0x7A, 0x00, 0x00},
 {0xC0, 0x40, 0x40, 0x7C, 0x42, 0x42, 0x42, 0x42, 0xFC, 0x00, 0x00},
 {0x00, 0x00, 0x00, 0x7C, 0x82, 0x80, 0x80, 0x82, 0x7C, 0x00, 0x00},
 {0x0C, 0x04, 0x04, 0x7C, 0x84, 0x84, 0x84, 0x84, 0x7E, 0x00, 0x00},
 {0x00, 0x00, 0x00, 0x7C, 0x82, 0xFE, 0x80, 0x82, 0x7C, 0x00, 0x00},
 {0x30, 0x40, 0x40, 0xF0, 0x40, 0x40, 0x40, 0x40, 0xF0, 0x00, 0x00},
 {0x00, 0x00, 0x00, 0x7E, 0x84, 0x84, 0x84, 0x7C, 0x04, 0x04, 0x78},
 {0xC0, 0x40, 0x40, 0x58, 0x64, 0x44, 0x44, 0x44, 0xEE, 0x00, 0x00},
 {0x08, 0x00, 0x00, 0x38, 0x08, 0x08, 0x08, 0x08, 0x3E, 0x00, 0x00},
 {0x08, 0x00, 0x00, 0x78, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x70},
 {0xC0, 0x40, 0x40, 0x4C, 0x48, 0x50, 0x70, 0x48, 0xC6, 0x00, 0x00},
 {0x30, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x7C, 0x00, 0x00},
 {0x00, 0x00, 0x00, 0xE8, 0x54, 0x54, 0x54, 0x54, 0xD6, 0x00, 0x00},
 {0x00, 0x00, 0x00, 0xD8, 0x64, 0x44, 0x44, 0x44, 0xEE, 0x00, 0x00},
 {0x00, 0x00, 0x00, 0x7C, 0x82, 0x82, 0x82, 0x82, 0x7C, 0x00, 0x00},
 {0x00, 0x00, 0x00, 0xFC, 0x42, 0x42, 0x42, 0x42, 0x7C, 0x40, 0xE0},
 {0x00, 0x00, 0x00, 0x7E, 0x84, 0x84, 0x84, 0x7C, 0x04, 0x0E, 0x00},
 {0x00, 0x00, 0x00, 0xEC, 0x32, 0x20, 0x20, 0x20, 0xF8, 0x00, 0x00},
 {0x00, 0x00, 0x00, 0x7C, 0x82, 0x70, 0x0C, 0x82, 0x7C, 0x00, 0x00},
 {0x00, 0x20, 0x20, 0x78, 0x20, 0x20, 0x20, 0x24, 0x18, 0x00, 0x00},
 {0x00, 0x00, 0x00, 0xCC, 0x44, 0x44, 0x44, 0x4C, 0x36, 0x00, 0x00},
 {0x00, 0x00, 0x00, 0xEE, 0x44, 0x44, 0x28, 0x28, 0x10, 0x00, 0x00},
 {0x00, 0x00, 0x00, 0xEE, 0x44, 0x54, 0x54, 0x28, 0x28, 0x00, 0x00},
 {0x00, 0x00, 0x00, 0xEE, 0x44, 0x38, 0x38, 0x44, 0xEE, 0x00, 0x00},
 {0x00, 0x00, 0x00, 0xEE, 0x44, 0x44, 0x28, 0x28, 0x10, 0x10, 0x60},
 {0x00, 0x00, 0x00, 0xFC, 0x88, 0x10, 0x20, 0x44, 0xFC, 0x00, 0x00},
 {0x0C, 0x10, 0x10, 0x10, 0x10, 0x60, 0x10, 0x10, 0x10, 0x10, 0x0C},
 {0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10},
 {0x60, 0x10, 0x10, 0x10, 0x10, 0x0C, 0x10, 0x10, 0x10, 0x10, 0x60},
 {0x00, 0x00, 0x62, 0x92, 0x8C, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }};

//Pointer to our font table
char* cour10_font = &cour10_font_array[0][0];
