#ifndef __DRAW_H
#define __DRAW_H

#include <io.h>
#include "system.h"
#include "lcd.h"
#include "alt_types.h"
#include "stdio.h"

#define DO_FILL 1
#define DO_NOT_FILL 0

#define FONT_10PT_ROW 11
#define FONT_10PT_COLUMN 8

#define SIZE1 1
#define SIZE2 2
#define SIZE3 3
#define SIZE4 4

#define MAX_VERTICES 20

//typedef int vga_frame_array[SCREENHEIGHT][SCREENWIDTH / 2];

//typedef struct {
//  vga_frame_array vga_frame;
//} vga_frame_buffer_struct;

typedef struct {
  int hbegin;
  int vbegin;
  int hend;
  int f_color;
  int b_color;
  char* string;
  char* font;
  int ms_delay;
  int ticks_at_last_move;
  int text_scroll_index;
  char text_scroll_started;
  int window_width;
  int length_of_string;
  int scroll_points;
  int string_points;
} vga_text_scroll_struct;

typedef struct {
  int x, y;
} point;

typedef struct {
  point start;
  point end;
  alt_32 slope;
} edge;

extern char* cour10_font;

//int vga_init_no_interrupt (int *controller_base, vga_frame_buffer_struct *buffer_base);

int vga_merge_colors(int red, int green, int blue);

//int vga_print_string(int horiz_offset, int vert_offset, int color, char *font, vga_frame_buffer_struct* vga_frame_buffer, char string[]);
int vga_print_string_8bit(int horiz_offset, int vert_offset, unsigned char index, char *font, int size, unsigned char* frameBuffer, char string[]);

//int vga_scroll_string(vga_text_scroll_struct* scroll, vga_frame_buffer_struct* vga_frame_buffer);

//vga_text_scroll_struct* vga_scroll_string_init(int hbegin, int vbegin, int hend, int f_color, int b_color, char* font, int ms_delay, char *string);

//void vga_scroll_string_quit(vga_text_scroll_struct* scroll);

//int vga_move_block(int xbegin, int ybegin, int xend, int yend, int x_distance, int y_distance, int backfill_color, vga_frame_buffer_struct* vga_frame_buffer);
int vga_move_block_8bit(int xbegin, int ybegin, int xend, int yend, int x_distance, int y_distance, unsigned char backfill_index, unsigned char* frameBuffer);

//int vga_print_char (int horiz_offset, int vert_offset, int color, char character, char *font, vga_frame_buffer_struct* vga_frame_buffer);
int vga_print_char_8bit (int horiz_offset, int vert_offset, unsigned char index, char character, char *font, int size, unsigned char* frameBuffer);

void vga_draw_line(int horiz_start, int vert_start, int horiz_end, int vert_end, int width, int color, short* frameBuffer);
void vga_draw_line_8bit(int horiz_start, int vert_start, int horiz_end, int vert_end, int width, unsigned char index, unsigned char* frameBuffer);

void vga_draw_polyline_8bit(point points[], int num_points, int width, unsigned char index, unsigned char* frameBuffer);

void vga_draw_polygon_8bit(point points[], alt_32 num_points, alt_32 width, alt_u8 index, alt_32 fill, alt_u8* frameBuffer);

void vga_swap_edges(edge *edge1, edge *edge2);

//short vga_get_pixel(int horiz, int vert, vga_frame_buffer_struct* vga_frame_buffer);

int vga_draw_circle_8bit(int Hcenter, int Vcenter, int radius, int width, unsigned char index, int fill, unsigned char* frameBuffer);

void vga_circle_points_8bit(int cx, int cy, int x, int y, unsigned char index, int fill, unsigned char* vga_frame_buffer);

int vga_draw_box (int Hstart, int Vstart, int Hend, int Vend, short color, int fill, short* frameBuffer);
int vga_draw_box_8bit (int Hstart, int Vstart, int Hend, int Vend, alt_32 width, unsigned char index, int fill, unsigned char* frameBuffer);

void vga_paint_block (int Hstart, int Vstart, int Hend, int Vend, short color, short* frameBuffer);
void vga_paint_block_8bit (int Hstart, int Vstart, int Hend, int Vend, unsigned char index, unsigned char* frameBuffer);

void vga_horiz_line (int start, int end, short color, short* frameBuffer);
void vga_horiz_line_8bit (int start, int end, alt_32 width, unsigned char index, unsigned char* frameBuffer);

void vga_vert_line (int start, int end, short color, short* frameBuffer);
static inline void vga_vert_line_8bit (int start, int end, alt_32 width, unsigned char index, unsigned char* frameBuffer);

static inline void vga_sort_line(int* start,int* end);

void vga_paint_screen (short colour, short* frameBuffer);
void vga_paint_screen_8bit (unsigned char index, unsigned char* frameBuffer);

/*******************************************************************************
*  Function: draw_hline
*
*  Purpose: Draw a horizontal line, two pixels at a time, from an even start
*           pixel to an odd end pixel.
* 
*******************************************************************************/
static inline void draw_hline(int start, int end, short color, short * frameBuffer) {
  int i;
  color = color & 0xffff;
  
  // Set up a pointer to words within the frame buffer
  alt_32* ptr = (alt_32*) frameBuffer;
  // start and end now index words instead of shorts
  start = start/2;
  end = end/2;
   
  for (i = start; i <= end; i++)
    ptr[i] = (color << 16) | color;
}

/*******************************************************************************
*  Function: draw_hline_8bit
*
*  Purpose: Draw a horizontal line to an 8 bit frame buffer, four pixels at a
*           time.
* 
*******************************************************************************/
static inline void draw_hline_8bit(int start, int end, unsigned char index, unsigned char * frameBuffer) {
  int i;
  index = index & 0xff;
  
  // Set up a pointer to words within the frame buffer
  alt_32* ptr = (alt_32*) frameBuffer;
  // start and end now index words instead of bytes
  start = start/4;
  end = end/4;
   
  for (i = start; i <= end; i++)
    ptr[i] = (index << 24) | (index << 16) | (index << 8) | index;
}

/******************************************************************
*  Function: vga_vert_line_8bit
*
*  Purpose: Draws a vertical line between and including two
*           pixels.
* 
*           NOTE: start must be less than end
* 
******************************************************************/
#ifdef GRAPHICS_2D_0_BASE

  static inline void vga_vert_line_8bit (int start, int end, alt_32 width, unsigned char index, unsigned char* frameBuffer) {
    vga_sort_line(&start, &end);
    alt_32 xstart, xend, ystart, yend;

    alt_u8* ptr = (alt_u8*)(((alt_u32)frameBuffer) & 0x7fffffff);
    
    ystart = xstart = (alt_u32)ptr + start - width/2;
    xend = (alt_u32)ptr + start + (width - width/2) - 1;
    yend = (alt_u32)ptr + end + (width - width/2) - 1;
    while (IORD(GRAPHICS_2D_0_BASE, 0x0) > 0) {
    }
    IOWR(GRAPHICS_2D_0_BASE, (0x10>>2), index);
    IOWR(GRAPHICS_2D_0_BASE, (0x20>>2), xstart);
    IOWR(GRAPHICS_2D_0_BASE, (0x24>>2), xend);
    IOWR(GRAPHICS_2D_0_BASE, (0x28>>2), ystart);
    IOWR(GRAPHICS_2D_0_BASE, (0x2c>>2), yend);
    IOWR(GRAPHICS_2D_0_BASE, 0x0, 0x2);
  }

#else

  static inline void vga_vert_line_8bit (int start, int end, alt_32 width, unsigned char index, unsigned char* frameBuffer) {
    vga_sort_line(&start, &end);
    alt_32 i, j;
    for (i = start; i <= end; i += SCREENWIDTH) 
      for (j = -width/2; j < (width - width/2); j++) {
        frameBuffer[i + j] = index;
      }
  }
  
#endif

/******************************************************************
*  Function: vga_set_pixel
*
*  Purpose: Sets the specified pixel to the specified color.
* 
*           [AC] frame buffers are now arrays
*
******************************************************************/
static inline void vga_set_pixel(int horiz, int vert, short color, short* frameBuffer)
{
  frameBuffer[vert*SCREENWIDTH+horiz] = color;
}

/******************************************************************
*  Function: vga_set_pixel_8bit
*
*  Purpose: Sets the specified pixel in 8 bit frame buffer to the
*           specified pallette index.
* 
*           [AC] frame buffers are now arrays
*
******************************************************************/
static inline void vga_set_pixel_8bit(int horiz, int vert, unsigned char index, unsigned char* frameBuffer)
{
  frameBuffer[vert*SCREENWIDTH+horiz] = index;
}


static inline void vga_sort_line(int* start,int* end) {
  int tmp;
  if (*end < *start) {
    tmp = *start;
    *start = *end;
    *end = tmp;
  }
}


//Color Definitions

#define ALICEBLUE             0xF7FF
#define ANTIQUEWHITE          0xFF7A
#define AQUA                  0x07FF
#define AQUAMARINE            0x7FFA
#define AZURE                 0xF7FF
#define BEIGE                 0xF7BB
#define BISQUE                0xFF38
// [AC] #define BLACK                 0x0020
#define BLACK                 0x0000
#define BLANCHEDALMOND        0xFF79
// [AC] #define BLUE                  0x003F
#define BLUE                  0x001F
#define BLUEVIOLET            0x897C
#define BROWN                 0xA165
#define BURLYWOOD             0xDDF0
#define CADETBLUE             0x5CF4
#define CHARTREUSE            0x7FE0
#define CHOCOLATE             0xD363
#define CORAL                 0xFBEA
#define CORNFLOWERBLUE        0x64BD
#define CORNSILK              0xFFFB
#define CRIMSON               0xD8A7
#define CYAN                  0x07FF
#define DARKBLUE              0x0031
#define DARKCYAN              0x0471
#define DARKGOLDENROD         0xBC21
#define DARKGRAY              0xAD75
#define DARKGREEN             0x0320
#define DARKKHAKI             0xBDAD
#define DARKMAGENTA           0x8831
#define DARKOLIVEGREEN        0x5365
#define DARKORANGE            0xFC60
#define DARKORCHID            0x99B9
#define DARKRED               0x8820
#define DARKSALMON            0xECAF
#define DARKSEAGREEN          0x8DF1
#define DARKSLATEBLUE         0x49F1
#define DARKSLATEGRAY         0x2A69
#define DARKTURQUOISE         0x067A
#define DARKVIOLET            0x903A
#define DEEPPINK              0xF8B2
#define DEEPSKYBLUE           0x05FF
#define DIMGRAY               0x6B6D
#define DODGERBLUE            0x1CBF
#define FELDSPAR              0xD4AE
#define FIREBRICK             0xB124
#define FLORALWHITE           0xFFFE
#define FORESTGREEN           0x2464
#define FUCHSIA               0xF83F
#define GAINSBORO             0xDEFB
#define GHOSTWHITE            0xFFFF
#define GOLD                  0xFEA0
#define GOLDENROD             0xDD24
#define GRAY                  0x8430
#define GRAY25                0x4228
#define GRAY50                0x7BEF
#define GRAY75                0xC638
// [AC] #define GREEN                 0x0420
#define GREEN                 0x07E0
#define GREENYELLOW           0xAFE5
#define HONEYDEW              0xF7FE
#define HOTPINK               0xFB76
#define INDIANRED             0xCAEB
#define INDIGO                0x4830
#define IVORY                 0xFFFE
#define KHAKI                 0xF731
#define LAVENDER              0xE73F
#define LAVENDERBLUSH         0xFFBE
#define LAWNGREEN             0x7FE0
#define LEMONCHIFFON          0xFFF9
#define LIGHTBLUE             0xAEFC
#define LIGHTCORAL            0xF430
#define LIGHTCYAN             0xE7FF
#define LIGHTGOLDENRODYELLOW  0xFFFA
#define LIGHTGREEN            0x9772
#define LIGHTGREY             0xD6BA
#define LIGHTPINK             0xFDB8
#define LIGHTSALMON           0xFD2F
#define LIGHTSEAGREEN         0x25B5
#define LIGHTSKYBLUE          0x867F
#define LIGHTSLATEBLUE        0x83BF
#define LIGHTSLATEGRAY        0x7473
#define LIGHTSTEELBLUE        0xB63B
#define LIGHTYELLOW           0xFFFC
#define LIME                  0x07E0
#define LIMEGREEN             0x3666
#define LINEN                 0xFFBC
// [AC] #define MAGENTA               0xF83F
#define MAGENTA               0xF81F
#define MAROON                0x8020
#define MEDIUMAQUAMARINE      0x6675
#define MEDIUMBLUE            0x0039
#define MEDIUMORCHID          0xBABA
#define MEDIUMPURPLE          0x93BB
#define MEDIUMSEAGREEN        0x3DAE
#define MEDIUMSLATEBLUE       0x7B7D
#define MEDIUMSPRINGGREEN     0x07F3
#define MEDIUMTURQUOISE       0x4EB9
#define MEDIUMVIOLETRED       0xC0B0
#define MIDNIGHTBLUE          0x18EE
#define MINTCREAM             0xF7FF
#define MISTYROSE             0xFF3C
#define MOCCASIN              0xFF36
#define NAVAJOWHITE           0xFEF5
#define NAVY                  0x0030
#define OLDLACE               0xFFBC
#define OLIVE                 0x8420
#define OLIVEDRAB             0x6C64
#define ORANGE                0xFD20
#define ORANGERED             0xFA20
#define ORCHID                0xDBBA
#define PALEGOLDENROD         0xEF75
#define PALEGREEN             0x9FF3
#define PALETURQUOISE         0xAF7D
#define PALEVIOLETRED         0xDBB2
#define PAPAYAWHIP            0xFF7A
#define PEACHPUFF             0xFEF7
#define PERU                  0xCC27
#define PINK                  0xFE39
#define PLUM                  0xDD3B
#define POWDERBLUE            0xB73C
#define PURPLE                0x8030
// [AC] #define RED                   0xF820
#define RED                   0xF800
#define ROSYBROWN             0xBC71
#define ROYALBLUE             0x00B2
#define SADDLEBROWN           0x8A22
#define SALMON                0xFC2E
#define SANDYBROWN            0xF52C
#define SEAGREEN              0x2C6A
#define SEASHELL              0xFFBD
#define SIENNA                0xA2A5
#define SILVER                0xC638
#define SKYBLUE               0x867D
#define SLATEBLUE             0x6AF9
#define SLATEGRAY             0x7432
#define SNOW                  0xFFFF
#define SPRINGGREEN           0x07EF
#define STEELBLUE             0x4436
#define TAN                   0xD5B1
#define TEAL                  0x0430
#define THISTLE               0xDDFB
#define TOMATO                0xFB28
#define TURQUOISE             0x473A
#define VIOLET                0xEC3D
#define VIOLETRED             0xD132
#define WHEAT                 0xF6F6
#define WHITE                 0xFFFF
#define WHITESMOKE            0xF7BE
#define YELLOW                0xFFE0
#define YELLOWGREEN           0x9E66

// Pallette indexes - all 100% pixel alpha
#define P_BLACK   0x0
#define P_TRANS   0x8
#define P_RED     0xf9
#define P_GREEN   0xfa
#define P_YELLOW  0xfb
#define P_BLUE    0xfc

#define P_HRED    0x1
#define P_HYELLOW 0x3
#define P_HBLUE   0x4

  
#endif /* __DRAW_H */

