
#include "lcd.h"
#include "auto32.h"
#include "alt_types.h"


void lcd_sync(void) {
  // wait for active period
  while ((IORD_LCD_SR & 0x100) == 0)
    ;
    
  // wait for blanking period
  while ((IORD_LCD_SR & 0x100) == 0x100)
    ;
}

void lcd_reset(void) {
  IOWR_LCD_CR(0);
  IOWR_LCD_DMA_CR(0);
  IOWR_LCD_INT_EN(0);
  IOWR_LCD_INT_CLR(0xffffffff);
  lcd.const_alpha1 = 0;
  lcd.const_alpha2 = 0;
  lcd.const_alpha3 = 0;
  lcd.const_alpha4 = 0;
  lcd.background = 0;
  lcd.win_l1_h_start = 0;
  lcd.win_l1_h_stop = 0;
  lcd.win_l1_v_start = 0;
  lcd.win_l1_v_stop = 0;
  lcd.win_l2_h_start = 0;
  lcd.win_l2_h_stop = 0;
  lcd.win_l2_v_start = 0;
  lcd.win_l2_v_stop = 0;
  lcd.win_l3_h_start = 0;
  lcd.win_l3_h_stop = 0;
  lcd.win_l3_v_start = 0;
  lcd.win_l3_v_stop = 0;
  lcd.win_l4_h_start = 0;
  lcd.win_l4_h_stop = 0;
  lcd.win_l4_v_start = 0;
  lcd.win_l4_v_stop = 0;
  lcd.l0_dma_start = 0;
  lcd.l0_dma_len = 0;
  lcd.l1_dma_start = 0;
  lcd.l1_dma_len = 0;
  lcd.l2_dma_start = 0;
  lcd.l2_dma_len = 0;
  lcd.l3_dma_start = 0;
  lcd.l3_dma_len = 0;
  lcd.l4_dma_start = 0;
  lcd.l4_dma_len = 0;
  IOWR_LCD_CONST_ALPHA1(lcd.const_alpha1);
  IOWR_LCD_CONST_ALPHA2(lcd.const_alpha2);
  IOWR_LCD_CONST_ALPHA3(lcd.const_alpha3);
  IOWR_LCD_CONST_ALPHA4(lcd.const_alpha4);
  IOWR_LCD_BACKGROUND(lcd.background);
  IOWR_LCD_WIN1_H_START(lcd.win_l1_h_start);
  IOWR_LCD_WIN1_H_START(lcd.win_l1_h_stop);
  IOWR_LCD_WIN1_V_START(lcd.win_l1_v_start);
  IOWR_LCD_WIN1_V_START(lcd.win_l1_v_stop);
  IOWR_LCD_WIN2_H_START(lcd.win_l2_h_start);
  IOWR_LCD_WIN2_H_START(lcd.win_l2_h_stop);
  IOWR_LCD_WIN2_V_START(lcd.win_l2_v_start);
  IOWR_LCD_WIN2_V_START(lcd.win_l2_v_stop);
  IOWR_LCD_WIN3_H_START(lcd.win_l3_h_start);
  IOWR_LCD_WIN3_H_START(lcd.win_l3_h_stop);
  IOWR_LCD_WIN3_V_START(lcd.win_l3_v_start);
  IOWR_LCD_WIN3_V_START(lcd.win_l3_v_stop);
  IOWR_LCD_WIN4_H_START(lcd.win_l4_h_start);
  IOWR_LCD_WIN4_H_START(lcd.win_l4_h_stop);
  IOWR_LCD_WIN4_V_START(lcd.win_l4_v_start);
  IOWR_LCD_WIN4_V_START(lcd.win_l4_v_stop);
  IOWR_LCD_L0_DMA_START(lcd.l0_dma_start);
  IOWR_LCD_L0_DMA_LEN(lcd.l0_dma_len);
  IOWR_LCD_L0_DMA_START(lcd.l1_dma_start);
  IOWR_LCD_L0_DMA_LEN(lcd.l1_dma_len);
  IOWR_LCD_L0_DMA_START(lcd.l2_dma_start);
  IOWR_LCD_L0_DMA_LEN(lcd.l2_dma_len);
  IOWR_LCD_L0_DMA_START(lcd.l3_dma_start);
  IOWR_LCD_L0_DMA_LEN(lcd.l3_dma_len);
  IOWR_LCD_L0_DMA_START(lcd.l4_dma_start);
  IOWR_LCD_L0_DMA_LEN(lcd.l4_dma_len);
};

void lcdIsr (void* context, alt_u32 id) {
  // Reset interrupt bit
  IOWR_LCD_INT_CLR(0x1);
  // is next buffer available?
/*********  
  if ((lcdOwn + 1)%3 != videoOwn) {
    // update ownership and point to next buffer
    lcdOwn = (lcdOwn + 1)%3;
    IOWR_LCD_L1_DMA_START((unsigned int) buffers[lcdOwn]);
    // Load parameters for next frame
    IOWR_LCD_CONST_ALPHA1(lcd.const_alpha1);
    IOWR_LCD_CONST_ALPHA2(lcd.const_alpha2);
  }
*///////////////////////////////////////////  
  // set frame flag
  lcdFlags = lcdFlags | 0x1;
}

// Enable layers
void lcdLayers(int layers) {
  IOWR_LCD_DMA_CR(layers);
  IOWR_LCD_CR(layers);
};

// Set up PiP in centre of screen
void PIPCentre(int width, int height) {
  lcd.win_l1_h_start = SCREENWIDTH/2 - width/2;
  lcd.win_l1_h_stop = lcd.win_l1_h_start + width;
  lcd.win_l1_v_start = SCREENHEIGHT/2 - height/2;
  lcd.win_l1_v_stop = lcd.win_l1_v_start + height;
};

int lcdDoPIP(int layer) {
  switch (layer) {
    case 1:
      IOWR_LCD_WIN1_H_START(lcd.win_l1_h_start);
      IOWR_LCD_WIN1_H_STOP(lcd.win_l1_h_stop);
      IOWR_LCD_WIN1_V_START(lcd.win_l1_v_start);
      IOWR_LCD_WIN1_V_STOP(lcd.win_l1_v_stop);
      return 0;
    case 2:
      IOWR_LCD_WIN2_H_START(lcd.win_l2_h_start);
      IOWR_LCD_WIN2_H_STOP(lcd.win_l2_h_stop);
      IOWR_LCD_WIN2_V_START(lcd.win_l2_v_start);
      IOWR_LCD_WIN2_V_STOP(lcd.win_l2_v_stop);
      return 0;
    case 3:
      IOWR_LCD_WIN3_H_START(lcd.win_l3_h_start);
      IOWR_LCD_WIN3_H_STOP(lcd.win_l3_h_stop);
      IOWR_LCD_WIN3_V_START(lcd.win_l3_v_start);
      IOWR_LCD_WIN3_V_STOP(lcd.win_l3_v_stop);
      return 0;
    case 4:
      IOWR_LCD_WIN4_H_START(lcd.win_l4_h_start);
      IOWR_LCD_WIN4_H_STOP(lcd.win_l4_h_stop);
      IOWR_LCD_WIN4_V_START(lcd.win_l4_v_start);
      IOWR_LCD_WIN4_V_STOP(lcd.win_l4_v_stop);
      break;
    default:
      return -1;
  }
  return 0;
};

int lcdDoDMA(int layer, short *frameBuffer, int length, int alpha) {
  switch (layer) {
    case 0:
      // layer 0 has no layer alpha
      IOWR_LCD_L0_DMA_START((unsigned int) frameBuffer);
      IOWR_LCD_L0_DMA_LEN(length);
      return 0;
    case 1:
      IOWR_LCD_CONST_ALPHA1(alpha);
      IOWR_LCD_L1_DMA_START((unsigned int) frameBuffer);
      IOWR_LCD_L1_DMA_LEN(length);
      return 0;
    default:
      return -1;
  }
};

int lcdDoDMA8(int layer, unsigned char *frameBuffer, int length, int alpha) {
  switch (layer) {
    case 2:
      IOWR_LCD_CONST_ALPHA2(alpha);
      IOWR_LCD_L2_DMA_START((unsigned int) frameBuffer);
      IOWR_LCD_L2_DMA_LEN(length);
      return 0;
    case 3:
      IOWR_LCD_CONST_ALPHA3(alpha);
      IOWR_LCD_L3_DMA_START((unsigned int) frameBuffer);
      IOWR_LCD_L3_DMA_LEN(length);
      return 0;
    case 4:
      IOWR_LCD_CONST_ALPHA4(alpha);
      IOWR_LCD_L4_DMA_START((unsigned int) frameBuffer);
      IOWR_LCD_L4_DMA_LEN(length);
      return 0;
    default:
      return -1;
  }
};

