#ifndef __LCD_H
#define __LCD_H

/*
 * lcd.h
 * 
 */
 
#include <io.h>
#include "system.h"


// ch add
#define SCREENWIDTH 320//640
#define SCREENHEIGHT 240//480
#define SCREENSIZE (SCREENWIDTH)*(SCREENHEIGHT)
#define FRAMEBUFFERSIZE (SCREENWIDTH)*(SCREENHEIGHT)
#define LCDSTRETCH 5120 /* 1.25*4096 */

#define MAXVIDEOALPHA 0x3f

extern volatile int lcdFlags;

typedef struct {
  int const_alpha1;
  int const_alpha2;
  int const_alpha3;
  int const_alpha4;
  int background;
  int win_l1_h_start;
  int win_l1_h_stop;
  int win_l1_v_start;
  int win_l1_v_stop;
  int win_l2_h_start;
  int win_l2_h_stop;
  int win_l2_v_start;
  int win_l2_v_stop;
  int win_l3_h_start;
  int win_l3_h_stop;
  int win_l3_v_start;
  int win_l3_v_stop;
  int win_l4_h_start;
  int win_l4_h_stop;
  int win_l4_v_start;
  int win_l4_v_stop;
  int l0_dma_start;
  int l0_dma_len;
  int l1_dma_start;
  int l1_dma_len;
  int l2_dma_start;
  int l2_dma_len;
  int l3_dma_start;
  int l3_dma_len;
  int l4_dma_start;
  int l4_dma_len;
} LCDCONTROLLER;

void lcd_sync(void);
void lcd_reset(void);
void lcdIsr (void* context, alt_u32 id);
void lcdLayers(int layers);
void PIPCentre(int width, int height);
int lcdDoPIP(int layer);
int lcdDoDMA(int layer, short *frameBuffer, int length, int alpha);
int lcdDoDMA8(int layer, unsigned char *frameBuffer, int length, int alpha);

// control
#define IORD_LCD_SR	IORD(LCD_CONTROLLER_0_BASE, 0)
#define IOWR_LCD_CR(data)	IOWR(LCD_CONTROLLER_0_BASE, 0, data)

#define IORD_LCD_CONST_ALPHA1	IORD(LCD_CONTROLLER_0_BASE, (0x4>>2))
#define IOWR_LCD_CONST_ALPHA1(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x4>>2), data)
#define IORD_LCD_CONST_ALPHA2	IORD(LCD_CONTROLLER_0_BASE, (0x8>>2))
#define IOWR_LCD_CONST_ALPHA2(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x8>>2), data)
#define IORD_LCD_CONST_ALPHA3	IORD(LCD_CONTROLLER_0_BASE, (0xc>>2))
#define IOWR_LCD_CONST_ALPHA3(data)	IOWR(LCD_CONTROLLER_0_BASE, (0xc>>2), data)
#define IORD_LCD_CONST_ALPHA4	IORD(LCD_CONTROLLER_0_BASE, (0x10>>2))
#define IOWR_LCD_CONST_ALPHA4(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x10>>2), data)
#define IORD_LCD_BACKGROUND	IORD(LCD_CONTROLLER_0_BASE, (0x14>>2))
#define IOWR_LCD_BACKGROUND(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x14>>2), data)

// Windowing (PinP)
#define IORD_LCD_WIN1_H_START	IORD(LCD_CONTROLLER_0_BASE, (0x18>>2))
#define IOWR_LCD_WIN1_H_START(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x18>>2), data)
#define IORD_LCD_WIN1_H_STOP	IORD(LCD_CONTROLLER_0_BASE, (0x1c>>2))
#define IOWR_LCD_WIN1_H_STOP(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x1c>>2), data)
#define IORD_LCD_WIN1_V_START	IORD(LCD_CONTROLLER_0_BASE, (0x20>>2))
#define IOWR_LCD_WIN1_V_START(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x20>>2), data)
#define IORD_LCD_WIN1_V_STOP	IORD(LCD_CONTROLLER_0_BASE, (0x24>>2))
#define IOWR_LCD_WIN1_V_STOP(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x24>>2), data)
#define IORD_LCD_WIN2_H_START	IORD(LCD_CONTROLLER_0_BASE, (0x28>>2))
#define IOWR_LCD_WIN2_H_START(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x28>>2), data)
#define IORD_LCD_WIN2_H_STOP	IORD(LCD_CONTROLLER_0_BASE, (0x2c>>2))
#define IOWR_LCD_WIN2_H_STOP(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x2c>>2), data)
#define IORD_LCD_WIN2_V_START	IORD(LCD_CONTROLLER_0_BASE, (0x30>>2))
#define IOWR_LCD_WIN2_V_START(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x30>>2), data)
#define IORD_LCD_WIN2_V_STOP	IORD(LCD_CONTROLLER_0_BASE, (0x34>>2))
#define IOWR_LCD_WIN2_V_STOP(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x34>>2), data)
#define IORD_LCD_WIN3_H_START	IORD(LCD_CONTROLLER_0_BASE, (0x38>>2))
#define IOWR_LCD_WIN3_H_START(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x38>>2), data)
#define IORD_LCD_WIN3_H_STOP	IORD(LCD_CONTROLLER_0_BASE, (0x3c>>2))
#define IOWR_LCD_WIN3_H_STOP(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x3c>>2), data)
#define IORD_LCD_WIN3_V_START	IORD(LCD_CONTROLLER_0_BASE, (0x40>>2))
#define IOWR_LCD_WIN3_V_START(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x40>>2), data)
#define IORD_LCD_WIN3_V_STOP	IORD(LCD_CONTROLLER_0_BASE, (0x44>>2))
#define IOWR_LCD_WIN3_V_STOP(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x44>>2), data)
#define IORD_LCD_WIN4_H_START	IORD(LCD_CONTROLLER_0_BASE, (0x48>>2))
#define IOWR_LCD_WIN4_H_START(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x48>>2), data)
#define IORD_LCD_WIN4_H_STOP	IORD(LCD_CONTROLLER_0_BASE, (0x4c>>2))
#define IOWR_LCD_WIN4_H_STOP(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x4c>>2), data)
#define IORD_LCD_WIN4_V_START	IORD(LCD_CONTROLLER_0_BASE, (0x50>>2))
#define IOWR_LCD_WIN4_V_START(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x50>>2), data)
#define IORD_LCD_WIN4_V_STOP	IORD(LCD_CONTROLLER_0_BASE, (0x54>>2))
#define IOWR_LCD_WIN4_V_STOP(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x54>>2), data)

// DMA Masters
#define IORD_LCD_L0_DMA_START	IORD(LCD_CONTROLLER_0_BASE, (0x58>>2))
#define IOWR_LCD_L0_DMA_START(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x58>>2), data)
#define IORD_LCD_L0_DMA_LEN	IORD(LCD_CONTROLLER_0_BASE, (0x5c>>2))
#define IOWR_LCD_L0_DMA_LEN(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x5c>>2), data)
#define IORD_LCD_L1_DMA_START	IORD(LCD_CONTROLLER_0_BASE, (0x60>>2))
#define IOWR_LCD_L1_DMA_START(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x60>>2), data)
#define IORD_LCD_L1_DMA_LEN	IORD(LCD_CONTROLLER_0_BASE, (0x64>>2))
#define IOWR_LCD_L1_DMA_LEN(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x64>>2), data)
#define IORD_LCD_L2_DMA_START	IORD(LCD_CONTROLLER_0_BASE, (0x68>>2))
#define IOWR_LCD_L2_DMA_START(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x68>>2), data)
#define IORD_LCD_L2_DMA_LEN	IORD(LCD_CONTROLLER_0_BASE, (0x6c>>2))
#define IOWR_LCD_L2_DMA_LEN(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x6c>>2), data)
#define IORD_LCD_L3_DMA_START	IORD(LCD_CONTROLLER_0_BASE, (0x70>>2))
#define IOWR_LCD_L3_DMA_START(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x70>>2), data)
#define IORD_LCD_L3_DMA_LEN	IORD(LCD_CONTROLLER_0_BASE, (0x74>>2))
#define IOWR_LCD_L3_DMA_LEN(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x74>>2), data)
#define IORD_LCD_L4_DMA_START	IORD(LCD_CONTROLLER_0_BASE, (0x78>>2))
#define IOWR_LCD_L4_DMA_START(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x78>>2), data)
#define IORD_LCD_L4_DMA_LEN	IORD(LCD_CONTROLLER_0_BASE, (0x7c>>2))
#define IOWR_LCD_L4_DMA_LEN(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x7c>>2), data)

#define IORD_LCD_DMA_CR	IORD(LCD_CONTROLLER_0_BASE, (0x80>>2))
#define IOWR_LCD_DMA_CR(data)	IOWR(LCD_CONTROLLER_0_BASE, (0x80>>2), data)
#define IORD_LCD_INT_EN IORD(LCD_CONTROLLER_0_BASE, (0x84>>2))
#define IOWR_LCD_INT_EN(data) IOWR(LCD_CONTROLLER_0_BASE, (0x84>>2), data)
#define IORD_LCD_INT_STAT IORD(LCD_CONTROLLER_0_BASE, (0x88>>2))
#define IOWR_LCD_INT_CLR(data) IOWR(LCD_CONTROLLER_0_BASE, (0x88>>2), data)

#endif /* __LCD_H */
