#include <stdio.h>
#include "auto32.h"
#include "system.h"
#include "lcd.h"
#include "alt_types.h"

#include <stdlib.h>
#include <unistd.h>
#include "draw.h"
#include "string.h"

int vga_term_init(alt_u32 xstart, alt_u32 ystart, alt_u32 columns, alt_u32 rows, alt_u32 font_size, alt_u8 back_index, alt_u8 text_index, alt_u32 border, alt_u8 border_index, alt_u8* frameBuffer) {
  alt_u32 i;
  terminal.rows = rows;
  terminal.columns = columns;
  terminal.back_index = back_index;
  terminal.text_index = text_index;
  terminal.font_size = font_size;
  terminal.xstart = xstart;
  terminal.ystart = ystart;
  terminal.xend = xstart + columns*FONT_10PT_COLUMN*font_size;
  terminal.yend = ystart + rows*FONT_10PT_ROW*font_size;
  terminal.border = border;
  terminal.border_index = border_index;
  terminal.frameBuffer = frameBuffer;
  terminal.font = cour10_font;

  // set up background and cursor
  vga_term_cls();
  vga_term_set_cursor(0, 0);
  
  // draw border
  if (terminal.border > 0) {
    for (i = 1; i <= terminal.border; i++) {
      vga_draw_box_8bit(terminal.xstart-i, terminal.ystart-i, terminal.xend + i, terminal.yend + i, 1, terminal.border_index, DO_NOT_FILL, terminal.frameBuffer);
    }
  }
  return (0);
}

// Set text pallette index
void vga_term_set_text(alt_u8 text_index){
  terminal.text_index = text_index;
}
 
// Set background pallette index 
void vga_term_set_back(alt_u8 back_index) {
  terminal.back_index = back_index;
}

// Set cursor position
void vga_term_set_cursor(alt_u32 cursorx, alt_u32 cursory) {
  terminal.cursorx = cursorx;
  terminal.cursory = cursory;
}

// get current cursor position
point vga_term_get_cursor(void) {
  point p;
  p.x = terminal.cursorx;
  p.y = terminal.cursory;
  return p;
}

// Clear terminal window and reset cursor position
void vga_term_cls() {
  // paint and set cursor pos
  vga_draw_box_8bit(terminal.xstart, terminal.ystart, terminal.xend, terminal.yend, 1, terminal.back_index, DO_FILL, terminal.frameBuffer);
  terminal.cursorx = 0;
  terminal.cursory = 0;
}  

void vga_term_print(char string[]) {
  int i = 0;
  // Print until we hit the '\0' char.
  while (string[i]) {
    // Handle newline char here.
    if (string[i] == '\n') {
      vga_term_newline();
      i++;
      continue;
    }
    // print a character and update cursor
    vga_print_char_8bit(terminal.xstart + terminal.cursorx*FONT_10PT_COLUMN*terminal.font_size,
                        terminal.ystart + terminal.cursory*FONT_10PT_ROW*terminal.font_size,
                        terminal.text_index, string[i], terminal.font, terminal.font_size,
                        terminal.frameBuffer);
    i++;
    if (++terminal.cursorx == terminal.columns) {
      vga_term_newline();
    }
  }
}

void vga_term_newline() {
  terminal.cursorx = 0;
  if (++terminal.cursory == terminal.rows) {
    // Move whole screen up one row
    terminal.cursory = terminal.rows-1;
    vga_move_block_8bit(terminal.xstart, terminal.ystart + FONT_10PT_ROW*terminal.font_size,
                        terminal.xend, terminal.yend,
                        0, -FONT_10PT_ROW*terminal.font_size,
                        terminal.back_index, terminal.frameBuffer);
  }
}
